package minecraft.player;

import minecraft.item.Food;
import minecraft.item.Item;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.Serial;
import java.io.Serializable;
import java.lang.reflect.Field;
import java.util.Objects;
import java.util.UUID;

/**
 * Represents a player character (Steve) in the Minecraft world.
 *
 * @author Juraj Petrik
 * @version 0.9
 */
public class Steve extends Character implements Comparable<Steve>, Serializable {

    private static final Logger logger = LoggerFactory.getLogger(Steve.class);

    @Serial
    private static final long serialVersionUID = 1L;

    private java.util.UUID uuid = java.util.UUID.randomUUID();
    private int hunger;
    private int maxHunger;
    private boolean godMode;
    private Item items;

    /**
     * Constructs a new Steve character with specified parameters.
     *
     * @param name         the name of the character
     * @param health       the initial health value
     * @param maxHealth    the maximum health value
     * @param attackPower  the attack power value
     * @param defensePower the defense power value
     * @param hunger       the initial hunger level
     * @param maxHunger    the maximum hunger level
     * @param godMode      whether god mode is enabled
     * @throws InvalidCharacterParamsException if any parameter is invalid (negative, zero, or hunger exceeds maxHunger)
     */
    public Steve(String name, int health, int maxHealth, int attackPower, int defensePower,
                 int hunger, int maxHunger, boolean godMode) throws InvalidCharacterParamsException {
        super(name, health, maxHealth, attackPower, defensePower);

        if (hunger <= 0 || maxHunger <= 0) {
            logger.error("Attempted to create Steve with invalid hunger values: hunger={}, maxHunger={}", hunger, maxHunger);
            throw new InvalidCharacterParamsException("Cannot be negative or zero!");
        }

        if (hunger > maxHunger) {
            logger.error("Attempted to create Steve with hunger > maxHunger: {} > {}", hunger, maxHunger);
            throw new InvalidCharacterParamsException("Hunger can not be larger than maxHunger");
        }

        this.setHunger(hunger);
        this.setMaxHunger(maxHunger);
        this.setGodMode(godMode);
        logger.info("Created new Steve character: {}", this);
    }

    /**
     * Consumes the specified food item, restoring health and hunger.
     * The food amount is decreased by 1 if consumption is successful.
     *
     * @param food the food item to consume
     */
    public void consume(Food food) {
        if (food.getAmount() > 0) {
            int oldHealth = getHealth();
            int oldHunger = getHunger();

            setHealth(Math.min(getHealth() + food.getHeal(), getMaxHealth()));
            setHunger(Math.min(getHunger() + food.getFeed(), getMaxHunger()));
            food.setAmount(food.getAmount() - 1);

            logger.info("{} consumed food: health {}->{}, hunger {}->{}, remaining food: {}",
                    getName(), oldHealth, getHealth(), oldHunger, getHunger(), food.getAmount());
        } else {
            logger.warn("{} attempted to consume food but none was available", getName());
        }
    }

    /**
     * Attacks a non-player character (NPC).
     * If god mode is enabled, the attack is instantly lethal.
     *
     * @param who the NPC to attack
     */
    @Override
    public void attack(NPC who) {
        int realAttackPower = godMode ? who.getMaxHealth() : Math.max(0, getAttackPower() - who.getDefensePower());
        who.setHealth(who.getHealth() - realAttackPower);

        logger.info("{} attacked {} with damage {} (godMode={})",
                getName(), who.getName(), realAttackPower, godMode);
    }

    /**
     * Attacks a Creeper character.
     * If god mode is enabled, the attack is instantly lethal and also triggers the Creeper's bomb timer.
     *
     * @param who the Creeper to attack
     */
    public void attack(Creeper who) {
        int realAttackPower = godMode ? who.getMaxHealth() : Math.max(0, getAttackPower() - who.getDefensePower());

        who.setHealth(who.getHealth() - realAttackPower);
        who.setBombTimer(1.5);

        logger.info("{} attacked Creeper {} with damage {} (godMode={}), bomb timer set to 1.5",
                getName(), who.getName(), realAttackPower, godMode);
    }

    /**
     * Attacks another Steve character.
     * The attack has no effect if the target has god mode enabled.
     *
     * @param who the Steve character to attack
     */
    public void attack(Steve who) {
        if (who.isGodMode()) {
            logger.info("{} attempted to attack {} but target has god mode enabled", getName(), who.getName());
            return;
        }

        int realAttackPower = godMode ? who.getMaxHealth() : Math.max(0, getAttackPower() - who.getDefensePower());
        who.setHealth(who.getHealth() - realAttackPower);

        logger.info("{} attacked {} with damage {} (godMode={})",
                getName(), who.getName(), realAttackPower, godMode);
    }

    /**
     * @return the current hunger level
     */
    public int getHunger() {
        return hunger;
    }

    /**
     * @param hunger the new hunger level
     */
    public void setHunger(int hunger) {
        logger.debug("{} hunger changed from {} to {}", getName(), this.hunger, hunger);
        this.hunger = hunger;
    }

    /**
     * @return the maximum hunger level
     */
    public int getMaxHunger() {
        return maxHunger;
    }

    /**
     * @param maxHunger the new maximum hunger level
     */
    public void setMaxHunger(int maxHunger) {
        logger.debug("{} maxHunger changed from {} to {}", getName(), this.maxHunger, maxHunger);
        this.maxHunger = maxHunger;
    }

    /**
     * @return true if god mode is enabled, false otherwise
     */
    public boolean isGodMode() {
        return godMode;
    }

    /**
     * @param godMode true to enable god mode, false to disable
     */
    public void setGodMode(boolean godMode) {
        logger.debug("{} god mode changed from {} to {}", getName(), this.godMode, godMode);
        this.godMode = godMode;
    }

    /**
     * Compares this Steve to another based on name (case-sensitive).
     *
     * @param o the other Steve to compare to
     * @return a negative integer, zero, or a positive integer as this Steve's name
     * is less than, equal to, or greater than the specified Steve's name
     */
    @Override
    public int compareTo(Steve o) {
        return o.getName().compareTo(this.getName());
    }


    @Override
    public boolean equals(Object object) {
        if (this == object) return true;
        if (!(object instanceof Steve steve)) return false;

        return Objects.equals(getName(), steve.getName()) && getHealth() == steve.getHealth()
                && getMaxHealth() == steve.getMaxHealth() && getAttackPower() == steve.getAttackPower()
                && getDefensePower() == steve.getDefensePower() && getHunger() == steve.getHunger()
                && getMaxHunger() == steve.getMaxHunger() && isGodMode() == steve.isGodMode();
    }

    @Override
    public int hashCode() {
        return Objects.hash(getName(), getHealth(), getMaxHealth(), getAttackPower(),
                getDefensePower(), getHunger(), getMaxHunger(), isGodMode());
    }

    @Override
    public String toString() {
        return String.format("Steve[name=%s, health=%d/%d, attack=%d, defense=%d, hunger=%d/%d, godMode=%b]",
                getName(), getHealth(), getMaxHealth(), getAttackPower(), getDefensePower(),
                getHunger(), getMaxHunger(), isGodMode());
    }
}